{% load i18n sizeformat parse_date %}

<h3>{% trans "Snapshot Overview" %}</h3>

<div class="info row-fluid detail">
  <h4>{% trans "Info" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ snapshot.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ snapshot.id }}</dd>
    <dt>{% trans "Source" %}</dt>
    <dd>{{ snapshot.share_id }}</dd>
    {% if snapshot.description %}
    <dt>{% trans "Description" %}</dt>
    <dd>{{ snapshot.description }}</dd>
    {% endif %}
    <dt>{% trans "Status" %}</dt>
    <dd>{{ snapshot.status|capfirst }}</dd>
  </dl>
</div>

<div class="specs row-fluid detail">
  <h4>{% trans "Specs" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Size" %}</dt>
    <dd>{{ snapshot.size }} {% trans "GiB" %}</dd>
    <dt>{% trans "Created" %}</dt>
    <dd>{{ snapshot.created_at|parse_date }}</dd>
  </dl>
</div>
